@@ -1,16 +1,25 @@
+import json
+import os
 from collections import OrderedDict
-
-from ..utils import import_attribute
+from concurrent.futures import ThreadPoolExecutor
 
 import six
 
+from ..utils import import_attribute, TimeAwareJSONEncoder
 
-__all__ = ('loaded_procs', 'Proc', 'load_proc', 'load_procs')
+__all__ = (
+    'loaded_procs', 'Proc', 'load_proc', 'load_procs', 'dump_procs',
+    'serialize_procs', 'ProcSerializer',
+)
+
+HIREFIRE_FOUND = 'HireFire Middleware Found!'
+USE_CONCURRENCY = os.environ.get('HIREFIRE_USE_CONCURRENCY', False)
 
 
 class Procs(OrderedDict):
     pass
 
+
 loaded_procs = Procs()
 
 
@@ -55,6 +64,57 @@ def load_procs(*procs):
     return loaded_procs
 
 
+class ProcSerializer(object):
+    """
+    Callable that transforms procs to dictionaries.
+
+    Maintains an instance cache that will be reused across calls.
+    """
+    def __init__(self):
+        self.cache = {}
+
+    def __call__(self, args):
+        name, proc = args
+        try:
+            quantity = proc.quantity(cache=self.cache)
+        except TypeError:
+            quantity = proc.quantity()
+        return {
+            'name': name,
+            'quantity': quantity or 0,
+        }
+
+
+def serialize_procs(procs, use_concurrency=USE_CONCURRENCY,
+                    serializer_class=ProcSerializer):
+    """
+    Given a list of loaded procs, serialize the data for them into
+    a list of dictionaries in the form expected by HireFire,
+    ready to be encoded into JSON.
+    """
+    serializer = serializer_class()
+
+    if use_concurrency:
+        with ThreadPoolExecutor() as executor:
+            # Execute all procs in parallel to avoid blocking IO
+            # especially celery which needs to open a transport to AMQP.
+            proc_iterator = executor.map(serializer, procs.items())
+    else:
+        proc_iterator = map(serializer, procs.items())
+
+    # Return a list, since json.dumps does not support generators.
+    return list(proc_iterator)
+
+
+def dump_procs(procs):
+    """
+    Given a list of loaded procs dumps the data for them in
+    JSON format.
+    """
+    data = serialize_procs(procs)
+    return json.dumps(data, cls=TimeAwareJSONEncoder, ensure_ascii=False)
+
+
 class Proc(object):
     """
     The base proc class. Use this to implement custom queues or
@@ -105,11 +165,18 @@ class Proc(object):
         return ("<Proc %s: '%s.%s'>" %
                 (self.name, cls.__module__, cls.__name__))
 
-    def quantity(self, *args, **kwargs):
+    def quantity(self, **kwargs):
         """
         Returns the aggregated number of tasks of the proc queues.
 
         Needs to be implemented in a subclass.
+
+        ``kwargs`` must be captured even when not used, to allow for
+        future extensions.
+
+        The only kwarg currently implemented is ``cache``, which is
+        a dictionary made available for cross-proc caching. It is
+        empty when the first proc is processed.
         """
         raise NotImplementedError
 
@@ -137,6 +204,7 @@ class ClientProc(Proc):
     class for an example.
 
     """
+
     def __init__(self, *args, **kwargs):
         super(ClientProc, self).__init__(*args, **kwargs)
         self.clients = []
